/* Include all Iguana header files */
#ifndef IGUANA_H
#define IGUANA_H

/* Since this is a "meta-include" file, no #ifdef __cplusplus extern "C" { */
#include "config.h"

/*
 * Iguana API
 * ----------
 *
 * All user visible names defined by this file (except those defined by the 
 * included standard headers) have one of the prefixes "Ig" or "_Ig". Names
 * beginning with "_Ig" are for internal use by the Iguana implementation
 * and should not be used by extension/emdedding writers. All symbols defined 
 * as public (via Ig_API; see also api.h) should begin with "Ig", to avoid
 * infringing on user name space. 
 *
 * C++ developers should note that though the API is defined entirely using C,
 * the header files do properly declare the entry points to be extern "C", so
 * there is no need to do anything special to use the API from C++.
 */

/*
 * Functions Failure Modes
 * -----------------------
 *
 * Functions may fail for a variety of reasons, including running out of
 * memory.  This is communicated to the caller in two ways: an error string
 * is set (see errors.h), and the function result differs: functions that
 * normally return a pointer return NULL for failure, functions returning
 * an integer return -1 (which could be a legal return value too!).
 *
 * Callers should-always-check for errors before using the result.
 */

/* API version and symbol visibility */
#include "api.h"

/* Platform-Independent Interfaces */
#include "system.h"

/* Abstract Objects Layer */
#include "object.h"
#include "abstract.h"

/* Concrete Objects Layer */
#include "arrayobject.h"
#include "boolobject.h"
#include "dictobject.h"
#include "floatobject.h"
#include "intobject.h"
#include "methodobject.h"
#include "moduleobject.h"
#include "noneobject.h"
#include "stringobject.h"
#include "tupleobject.h"

/* Utilities */
#include "debug.h"
#include "future.h"
#include "version.h"

/* The Parser */
#include "node.h"
#include "parser.h"

/* The Interpreter */
#include "errors.h"
#include "eval.h"
#include "traceback.h"

/* Modules */
#include "import.h"
#include "modsupport.h"

/* The Very High Level Layer */
#include "iguanarun.h"

#endif /* IGUANA_H */
