/* Array object interface */

/* 
 * This is a mutable type: the array items can be changed, but not their
 * number. Out-of-range indices or non-array objects are detected.
 *
 * The components of a array must be of the same type. Examples of vectors 
 * in Iguana notation are [1,2], ["q","w","e","r","t","y"]. The degenerate 
 * array with zero components, written [], is known as the unit type since it
 * has only one possible value which is also written [].
 */

#ifndef Ig_ARRAYOBJECT_H
#define Ig_ARRAYOBJECT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* This subtype of IgObject represents an Iguana array object. */
typedef struct _IgArrayObject IgArrayObject;

struct _IgArrayObject
{
	IgObject_VAR_HEAD;
	IgObject **ob_data;
};

/* This instance of IgTypeObject represents the Iguana array type. */
Ig_API(IgTypeObject) IgArray_Type;

/* Return true if its argument is an Iguana array object. */
#define IgArray_TYPE(obj) 	((obj)->ob_type == &IgArray_Type)

/* Cast the object pointer passed as argument to a IgArrayObject pointer. */
#define IgArray_CAST(obj)	((IgArrayObject *) (obj))

/* 
 * Macros for direct access to the array values. Type checks as well as 
 * out-of-range indices checks are *not* done, so use with care. 
 */
#define IgArray_SIZE(obj)	(IgArray_CAST(obj)->ob_size)
#define IgArray_ITEM(obj,i)	(IgArray_CAST(obj)->ob_data[i])

/* Return a new array object of size passed as argument. */
Ig_API(IgObject *) IgArray_FromSize(const int); 

/* Return a copy of the array. */
Ig_API(IgObject *) IgArray_Dup(IgObject *);

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_ARRAYOBJECT_H */
