/* Debugging macros and functions */
#ifndef Ig_DEBUG_H
#define Ig_DEBUG_H

#define Ig_STRINGIFY(macro_or_string)    Ig_STRINGIFY_ARG (macro_or_string)
#define Ig_STRINGIFY_ARG(contents)       #contents

/* Provide a string identifying the current code position */
#if defined(__GNUC__) && (__GNUC__ < 3)
# define Ig_STRLOC \
  "" __FILE__ ": " Ig_STRINGIFY (__LINE__) " in " __PRETTY_FUNCTION__ "()" ""
#else
# define Ig_STRLOC \
  "" __FILE__ ": " Ig_STRINGIFY (__LINE__) ""
#endif

#ifdef Ig_DEBUG
/* uncomment to enable Ig_ASSERT and Ig_ASSERT_NOT_REACHED macros */
# define Ig_WITH_ASSERT

/* uncomment to enable yyparse() debug */
/* # define Ig_WITH_PARSER_DEBUG */
#endif

/* 
 * Provide macros for error handling. The "assert" macros will exit on 
 * failure. The "return" macros will exit the current function. Two 
 * different definitions are given for the macros if Ig_WITH_ASSERT 
 * is defined, in order to support gcc's __PRETTY_FUNCTION__ capability.
 */

#ifndef Ig_WITH_ASSERT
# define Ig_ASSERT(expr)
# define Ig_ASSERT_NOT_REACHED()
#else
# define Ig_ASSERT(expr) \
  if (!(expr)) {							\
	Ig_Suicide("" Ig_STRLOC ": assertion `" #expr "' failed");	\
  }
# define Ig_ASSERT_NOT_REACHED() \
	Ig_Suicide("" Ig_STRLOC ": should not be reached");
#endif

#if defined(__cplusplus)
extern "C"
{
#endif

/* Utilities and gdb hooks (see also Objects/object.c and Misc/gdbinit). */
Ig_API(void) _IgObject_DumpAllocStatistics(void);
Ig_API(void) _IgObject_Inspect(IgObject *);

#if defined(__cplusplus)
}                               /* extern "C" { */
#endif

#endif /* Ig_DEBUG_H */
