/* Dict object interface */

/*
 * A dictionary that keeps a table of key-value (string-object) records. 
 * This is a mutable type: the dict entries can be changed and also their 
 * number (this is done internally).
 * 
 * NOTE: this type is not used in the language grammar, is intented only
 *       for internal use only.
 */

#ifndef Ig_DICTOBJECT_H
#define Ig_DICTOBJECT_H

#if defined(__cplusplus)
extern "C"
{
#endif

typedef struct _IgDictEntry IgDictEntry;

struct _IgDictEntry
{
	unsigned int dce_hash;	/* cached hash code of dce_key */
	IgObject *dce_key;	/* the search key */
	IgObject *dce_value;	/* the entry value */
	IgDictEntry *dce_next;	/* pointer to the next entry */
};

/* This subtype of IgObject represents an Iguana dict object. */
typedef struct _IgDictObject IgDictObject;

struct _IgDictObject
{
	IgObject_VAR_HEAD;
	int ob_used;		/* #slots used */
	IgDictEntry **ob_data;	/* slots */
};

/* This instance of IgTypeObject represents the Iguana dict type. */
Ig_API(IgTypeObject) IgDict_Type;

/* Return true if its argument is an Iguana dict object. */
#define IgDict_TYPE(obj) 	((obj)->ob_type == &IgDict_Type)

/* Cast the object pointer passed as argument to a IgDictObject pointer. */
#define IgDict_CAST(obj)        ((IgDictObject *)(obj))

/* Return an Iguana dictionary */
Ig_API(IgObject *) IgDict_New(void);	/* size here (implicitly) is 11 */
Ig_API(IgObject *) IgDict_NewWithSize(int);

/* Dictionary  */ 
Ig_API(int) IgDict_GetItem(IgObject *, const char *, IgObject **);
Ig_API(int) IgDict_SetItem(IgObject *, const char *, IgObject *);
Ig_API(int) IgDict_DelItem(IgObject *, const char *);

/* Return an array the array of keys */
Ig_API(IgObject *) IgDict_Keys(IgObject *);

/* Return an array the array of values */
Ig_API(IgObject *) IgDict_Values(IgObject *);

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_DICTOBJECT_H */
