/* Error handling interface */
#ifndef Ig_ERRORS_H
#define Ig_ERRORS_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* Error handling functions. */
Ig_API(void) IgErr_SetFromObject(IgObject *);
Ig_API(void) IgErr_SetFromString(IgObject *, const char *);
Ig_API(void) IgErr_SetFromFormat(IgObject *, const char *, ...)
                   Ig_ATTRIBUTE((format(printf, 2, 3)));
Ig_API(void) IgErr_SetFromErrno(IgObject *, const char *);

/* 
 * Test whether the error indicator is set. If set, return the
 * exception type (the first argument to the last call to one of
 * the IgErr_Set*() functions.
 */
Ig_API(IgObject *) IgErr_IsOccurred(void);

/* 
 * Clear the error indicator. If the error indicator is not set,
 * there is no effect. 
 */
Ig_API(void) IgErr_Clear(void);

/* Predefined errors. */
Ig_API(IgObject *) IgExc_AssertionError;
Ig_API(IgObject *) IgExc_DomainError;
Ig_API(IgObject *) IgExc_KeyboardInterrupt;
Ig_API(IgObject *) IgExc_ImportError;
Ig_API(IgObject *) IgExc_IndexError;
Ig_API(IgObject *) IgExc_InternalError;
Ig_API(IgObject *) IgExc_NameError;
Ig_API(IgObject *) IgExc_OverflowError;
Ig_API(IgObject *) IgExc_RuntimeError;
Ig_API(IgObject *) IgExc_SyntaxError;
Ig_API(IgObject *) IgExc_SystemError;
Ig_API(IgObject *) IgExc_TypeError;
Ig_API(IgObject *) IgExc_ValueError;
Ig_API(IgObject *) IgExc_ZeroDivisionError;

/* Predefined warnings. */
Ig_API(IgObject *) IgExc_DeprecationWarning;
Ig_API(IgObject *) IgExc_RuntimeWarning;
Ig_API(IgObject *) IgExc_SyntaxWarning;

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_ERRORS_H */
