/* Float object interface */

/* Float object represents a double precision floating point number. */

#ifndef Ig_FLOATOBJECT_H
#define Ig_FLOATOBJECT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* This subtype of IgObject represents an Iguana floating point object. */
typedef struct _IgFloatObject IgFloatObject;

struct _IgFloatObject
{
	IgObject_HEAD;
	double ob_fval;
};

/* This instance of IgTypeObject represents the Iguana floating point type. */
Ig_API(IgTypeObject) IgFloat_Type;

/* Return true if its argument is an Iguana float object. */
#define IgFloat_TYPE(obj) 	((obj)->ob_type == &IgFloat_Type)

/* Cast the object pointer passed as argument to a IgFloatObject pointer. */
#define IgFloat_CAST(obj)	((IgFloatObject *) (obj))

/* 
 * Returns the C double value of the Iguana float object. Type checks 
 * are *NOT* done, so use with care. 
 */
#define IgFloat_VALUE(obj)      ((IgFloat_CAST(obj))->ob_fval)

/* Return Iguana float from C double. */
Ig_API(IgObject *) IgFloat_FromDouble(double);

/* Return Iguana float from C string. */
Ig_API(IgObject *) IgFloat_FromString(const char *);

/* 
 * Commonly used float objects (shared); probably also 0.5 and -1 as well as 
 * few others should be added here. 
 */
#define Ig_Zero		((IgObject *) &_Ig_ZeroStruct)
#define Ig_Half		((IgObject *) &_Ig_HalfStruct)
#define Ig_One		((IgObject *) &_Ig_OneStruct)
#define Ig_Two		((IgObject *) &_Ig_TwoStruct)

/* XXX: don't use these directly. */
Ig_API(IgFloatObject) _Ig_ZeroStruct;
Ig_API(IgFloatObject) _Ig_HalfStruct;
Ig_API(IgFloatObject) _Ig_OneStruct;
Ig_API(IgFloatObject) _Ig_TwoStruct;

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_FLOATOBJECT_H */
