/* Module definition and import interface */
#ifndef Ig_IMPORT_H
#define Ig_IMPORT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* 
 * Import a dynamic module. This is the equivalent of the Iguana expression:
 * import name
 */
Ig_API(int) IgImport_ImportModule(const char *);

#if 0
/* 
 * Import a symbol from a dynamic module. This is the equivalent of the Iguana 
 * expression: from name import name
 */  
#define IgImport_ImportFrom(m,a)  IgImport_ImportFromEx((m),(a),NULL)

/*
 * Import a symbol from a dynamic module and rename it. This is the 
 * equivalent of the Iguana expression: from name import name as name
 */
Ig_API(int) IgImport_ImportFromEx(const char *, const char *, const char *);
#endif

/*  
 * Convenience API for adding/getting/deleting a module name. Note that each 
 * module name implicitly creates a ``namespace''.
 */
Ig_API(IgObject *) IgImport_AddModule(const char *);
Ig_API(IgObject *) IgImport_GetModule(const char *);
Ig_API(int) IgImport_DelModule(const char *);

/* Return an array containing all the modules names */
Ig_API(IgObject *) IgImport_GetModulesNames(void);

/* 
 * Structure describing a single entry in the list of pre-loaded modules. 
 * Each of these structures gives the name and initialization function 
 * for a module built into the interpreter. Programs which embed Iguana 
 * may use an array of these structures in conjunction with 
 * IgImport_ExtendInittab() to provide additional built-in modules. 
 */

typedef int (*_IgImport_InitFunc) (void); 

struct inittab
{
	char *name;
	_IgImport_InitFunc initfunc;
};

/*
 * Add a single module to the existing table of built-in modules. This is 
 * a convenience wrapper around IgImport_ExtendInittab(), returning -1 if 
 * the table could not be extended. The new module can be imported by the 
 * name, and uses the function initfunc as the initialization function 
 * called on the first attempted import. 
 * 
 * This should be called before Ig_Initialize().
 */
Ig_API(int) IgImport_AppendInitTable(char *, _IgImport_InitFunc);

/* 
 * Add a collection of modules to the table of built-in modules. The newtab 
 * array must end with a sentinel entry which contains NULL for the name 
 * field; failure to provide the sentinel value can result in a memory fault. 
 * Returns 0 on success or -1 if insufficient memory could be allocated to 
 * extend the internal table. In the event of failure, no modules are added 
 * to the internal table. 
 *
 * This should be called before Ig_Initialize().
 */
Ig_API(int) IgImport_ExtendInitTable(struct inittab *);

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_IMPORT_H */
