/* Integer object interface */

/* IgIntObject represents a long precision number. */

#ifndef Ig_INTOBJECT_H
#define Ig_INTOBJECT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* This subtype of IgObject represents an Iguana integer object. */
typedef struct _IgIntObject IgIntObject;

struct _IgIntObject
{
	IgObject_HEAD;
	long ob_ival;
};

/* This instance of IgTypeObject represents the Iguana integer type. */
Ig_API(IgTypeObject) IgInt_Type;

/* Return true if its argument is an Iguana integer object. */
#define IgInt_TYPE(obj)      	((obj)->ob_type == &IgInt_Type)

/* Cast the object pointer passed as argument to a IgIntObject pointer. */
#define IgInt_CAST(obj)		((IgIntObject *)(obj))

/*
 * Returns the C long value of the Iguana integer object. Type checks
 * are *NOT* done, so use with care.
 */
#define IgInt_VALUE(obj)       	((IgInt_CAST(obj))->ob_ival)

/* Return Iguana integer from C long. */
Ig_API(IgObject *) IgInt_FromLong(long);

/* Return Iguana integer from C string. The base here is implicitly 10. */
Ig_API(IgObject *) IgInt_FromString(const char *);

/* 
 * Return Iguana integer according to the given base, which must be between 
 * 2 and 36 inclusive, from C string. 
 */
Ig_API(IgObject *) IgInt_FromStringAndBase(const char *, const int);

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_INTOBJECT_H */
