/* Method object interface */
#ifndef Ig_METHODOBJECT_H
#define Ig_METHODOBJECT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* Prototype for methods implemented in C */
typedef IgObject *(*IgCFunction) (IgObject *); 

/* This subtype of IgObject represents an Iguana method object. */
typedef struct _IgMethodObject IgMethodObject;

struct _IgMethodObject
{
	IgObject_HEAD;
	IgObject *me_name;
	IgCFunction me_meth;
	IgObject *me_doc;
	IgObject *me_module;
};

Ig_API(IgTypeObject) IgMethod_Type;

/* Return true if its argument is an Iguana method object. */
#define IgMethod_TYPE(obj) 	((obj)->ob_type == &IgMethod_Type)

/* Cast the object pointer passed as argument to a IgMethodObject pointer. */
#define IgMethod_CAST(obj)	((IgMethodObject *) (obj))

/*
 * Macros for direct access to the method values. Type checks are *not*
 * done, so use with care.
 */
#define IgMethod_NAME(obj)       (IgMethod_CAST(obj)->me_name)
#define IgMethod_DOC(obj)        (IgMethod_CAST(obj)->me_doc)

/* Return a new method object. */
Ig_API(IgObject *) IgMethod_New(const char *, IgCFunction, const char *, IgObject *);

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_METHODOBJECT_H */
