/* Module support interface */

/* 
 * These functions are useful when creating your own extensions modules 
 * and methods. 
 */

#ifndef Ig_MODSUPPORT_H
#define Ig_MODSUPPORT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* For unused parameters -- avoid a compiler warning */
#define Ig_UNUSED(v)    	(void) v

/* Define the Ig_DOC macro for in-line documentation */
#define Ig_DOC(name,str)    	static char name[] = (str)

/*
 * Parse the parameters of a C method that takes only *positional* 
 * parameters into local variables. Returns 0 on success; otherwise 
 * returns -1 and raises the appropriate exception.
 * 
 * Ig_GetArgs() parameters:
 *  - the parameters list (as tuple, None if there is no parameters)
 *  - the format string
 *  - the parameters (that must be of the type specified from the format
 *    string, see below). Pass them as reference.
 *
 * Supported format characters:
 *   'a' for array object 		
 *   'd' for dict object		
 *   'd' for float object	
 *   'i' for int object		
 *   'M' for module object		
 *   'm' for method object		
 *   'N' for numeric object		
 *   'n' for none (no parameters, the args list must equal to Ig_None)	
 *   'o' for generic object		
 *   'S' for sequential object		
 *   's' for string object		
 *   'T' for tuple object		
 */
Ig_API(int) Ig_GetArgs(IgObject *, char *, ...);

/* Module's method table */
typedef struct _IgMethodDef IgMethodDef;

/* Method list */
struct _IgMethodDef
{
	char *ml_name;
	IgCFunction ml_meth;
	char *ml_doc;
};

#define Ig_InitModule(name,methods,doc) \
	Ig_InitModuleEx(name,methods,doc,Ig_API_VERSION)

Ig_API(IgObject *) Ig_InitModuleEx(const char *, IgMethodDef *,
                                     const char *, int);

/* Convenience functions - set a module attribute from a C constant. */
Ig_API(int) Ig_SetIntAttr(IgObject *, const char *, long); 
Ig_API(int) Ig_SetFloatAttr(IgObject *, const char *, double); 
Ig_API(int) Ig_SetStringAttr(IgObject *, const char *, const char *); 

/* Set a module attribute from an Iguana object. */
Ig_API(int) Ig_SetAttr(IgObject *, const char *, IgObject *); 

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_MODSUPPORT_H */
