/* String object interface */

/*
 * IgStringObject represents a character string.  An extra zero byte is
 * reserved at the end to ensure it is zero-terminated, but a size is
 * present so strings with null bytes in them can be represented.  This
 * is an immutable object type.
 *
 * There are functions to create new string objects, to test an object for 
 * string-ness, and to get the string value.  
 */

#ifndef Ig_STRINGOBJECT_H
#define Ig_STRINGOBJECT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* This subtype of IgObject represents an Iguana string object. */
typedef struct _IgStringObject IgStringObject;

struct _IgStringObject
{
	IgObject_VAR_HEAD;
	char *ob_sval;
};

/* This instance of IgTypeObject represents the Iguana string type. */
Ig_API(IgTypeObject) IgString_Type;

/* Return true if its argument is an Iguana string object. */
#define IgString_TYPE(obj) 	 ((obj)->ob_type == &IgString_Type)

/* Cast the object pointer passed as argument to a IgStringObject pointer. */
#define IgString_CAST(obj)       ((IgStringObject *) (obj))

/*
 * Macros for direct access to the string values. Type checks are *not*
 * done, so use with care.
 */
#define IgString_SIZE(obj)       (IgString_CAST(obj)->ob_size)
#define IgString_VALUE(obj)      (IgString_CAST(obj)->ob_sval)

/* Returns Iguana strings from C string. */
Ig_API(IgObject *) IgString_FromString(const char *);
Ig_API(IgObject *) IgString_FromStringAndSize(const char *, const int);
Ig_API(IgObject *) IgString_FromFormat(const char *, ...)
                             Ig_ATTRIBUTE((format(printf, 1, 2)));
Ig_API(IgObject *) IgString_FromVaList(const char *, va_list)
                             Ig_ATTRIBUTE((format(printf, 1, 0)));

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_STRINGOBJECT_H */
