/* Tuple object interface */

/* 
 * Another generally useful object type is an tuple of object pointers.
 * This is a mutable type: the tuple items can be changed, but not their
 * number. Out-of-range indices or non-tuple objects are detected.
 *
 * The components of a tuple may be of different types. Examples of tuples 
 * in Iguana notation are (1,2), ("Tuple",True), (w,(x,y),z). The degenerate 
 * tuple with zero components, written (), is known as the unit type since it
 * has only one possible value which is also written ().
 */

#ifndef Ig_TUPLEOBJECT_H
#define Ig_TUPLEOBJECT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* This subtype of IgObject represents an Iguana tuple object. */
typedef struct _IgTupleObject IgTupleObject;

struct _IgTupleObject
{
	IgObject_VAR_HEAD;
	IgObject **ob_data;
};

/* This instance of IgTypeObject represents the Iguana tuple type. */
Ig_API(IgTypeObject) IgTuple_Type;

/* Return true if its argument is an Iguana tuple object. */
#define IgTuple_TYPE(obj)       ((obj)->ob_type == &IgTuple_Type)

/* Cast the object pointer passed as argument to a IgTupleObject pointer. */
#define IgTuple_CAST(obj)       ((IgTupleObject *) (obj))

/*
 * Macros for direct access to the tuple values. Type checks are *not*
 * done, so use with care.
 */
#define IgTuple_SIZE(obj)       (IgTuple_CAST(obj)->ob_size)
#define IgTuple_ITEM(obj,i)     (IgTuple_CAST(obj)->ob_data[i])

/* Return a new tuple object of size passed as argument. */
Ig_API(IgObject *) IgTuple_FromSize(const int); 

/* Return a copy of the tuple. */
Ig_API(IgObject *) IgTuple_Dup(IgObject *);

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_TUPLEOBJECT_H */
